//******************************************************************************
// Programa TERMOMETRO.C - Termometro para medir duas temperaturas (uso diverso)
//
// Desenvolvido por Mrcio Jos Soares
// para Revista Saber Eletrnica
//
// Usa a INT do TIMER0 para ler as duas temperaturas
//
// Usa a INT do TIMER1 para verificar se temps atingiram valor mximo
//
// Usa lao para inserir dados em displays de sete segmentos
//
// Verso trabalha com Watch Dog Timer ligado
//
// Microcontrolador utilizado: PIC16F819-I/P
// Compilador: CCS C Compiler - verso PCM 3.15 - IDE verso 3.28
//
// Pinos utilizados:
// RA0 - AD 1 (entrada para sensores LM35 aps amplificao)
// RA1 - seleciona qual LM35 ser lido atravs do CD4051
// RA2 - controle catodo DISP4
// RA3 - controle catodo DISP3
// RA4 - controle catodo DISP2
// RA5 - jumper JP1
// RA6 - controle LED1 (indica que preset para temp1 alcanada)
// RA7 - controle LED2 (indica que preset para temp2 alcanada)
// RB0 - controle catodo DISP1
// RB1 - segmento A para displays
// RB2 - segmento B para displays
// RB3 - segmento C para displays
// RB4 - segmento D para displays
// RB5 - segmento E para displays
// RB6 - segmento F para displays
// RB7 - segmento G para displays
//
// Obs: utilizado LM358 para elevar sinal do LM35
//
//******************************************************************************

//******************************************************************************
// Este cdigo-fonte  parte integrante do artigo Termometro para medir duas
// temperaturas com microcontrolador PIC. Maiores informaes sobre o projeto
// podem ser obtidas no referido artigo publicado na Revista Saber Eletrnica
//******************************************************************************

//******************************************************************************
// O Copyright deste programa est reservado para Mrcio Jos Soares e seu
// contedo est protegido pela Lei de Direitos Autorais LEI N 9.610,
// de 19 de Fevereiro de 1998.  estritamente proibida o uso comercial como
// tambm a publicao total ou parcial dos contedos aqui apresentados
// sem prvia autorizao, por escrito, do autor.
//******************************************************************************

//******************************************************************************
// Arquivos incluidos
//******************************************************************************
#include "termometro.h"    //diretrizes de pr-compilao para o mdulo

//******************************************************************************
// Funes do mdulo
//******************************************************************************

//******************************************************************************
// Funo adapta_valor - insere um valor de 16bits em duas variveis de 8 bits
// recebe -  valor a ser inserido e posio (0 - DISP1 e 2 ; 1 - DISP 3 e 4)
// retorna - nada (retorno dentro de varivel global)
//******************************************************************************
void adapta_valor(int16 valor,unsigned char posi){

   unsigned char adapt;

   adapt = (unsigned char)valor;

   if(!posi){                        //grava valores para DISP1 e DISP2
      valor_disp[0] = adapt / 10;    //grava primeiro digito
      valor_disp[1] = adapt % 10;    //grava segundo digito
   }
   else{                             //grava valores para DISP3 e DISP4
      valor_disp[2] = adapt / 10;    //grava primeiro digito
      valor_disp[3] = adapt % 10;    //grava segundo digito
   }
}

//******************************************************************************
// Funo transforma - transforma um valor ponto flutuante em um inteiro 16 bits
// recebe -  valor (leitura feita)
// retorna - inteiro de 16 bits
//******************************************************************************
int16 transforma(int32 valor){

	union converte{
		int32 bigvalue;
		int16 midlevalue;
	}newvalue;

	newvalue.bigvalue=valor;
	restart_WDT();											//zera WDT
	return (newvalue.midlevalue);
}

//******************************************************************************
// Funo le_AD - le canal AD0 e retorna valor em formato inteiro 16bits
// recebe - nada
// retorna - 	inteiro de 16 bits
//******************************************************************************
int16 le_AD(void){

	int16 valor;
	int32 converte;

	restart_WDT();      								//zera WDT
	valor=read_adc();									//le canal analgico
	if (valor)											//se valor maior que 1, soma 1
		valor += 1;

	converte=(((int32)valor * 4882)/10000);	//ajusta convero 10bits 5/1024 = 0,004882
	valor=transforma(converte);					//pega valor 32 bits, converte para 16bits
	restart_WDT();										//zera WDT
	return(valor);
}

//******************************************************************************
// Funo para tratar a interrupo do timer0
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer0
void trata_int_t0(){

   static int contador;

   restart_WDT();							//zera WDT

   //reinicia o timer0 em 131, mas a contagem que j passou
   set_timer0(131+get_timer0());
   contador++;
   //verifica se j ocorreram 125 interrupes
   //125 Hz / 62 = 1 Hz = 1/1 = 1 segundo
   if (contador == 125){
      contador = 0;
      output_low(L4066);           //prepara para ler primeiro canal
      valorADC=le_AD();				  //l dado do canal AD
      valorADC=valorADC/5;         //converte o valor lido para C
      adapta_valor(valorADC,0);    //insere valor nas variveis p/ displays

      output_high(L4066);          //prepara para ler segundo canal
      valorADC=le_AD();				  //l dado do canal AD
      valorADC=valorADC/5;         //converte o valor lido para C
      adapta_valor(valorADC,1);    //insere valor nas variveis p/ displays
   }

   restart_WDT();							//zera WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer1
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer1
void trata_int_t1(){

	static int conta;
	static boolean s1;
  	static boolean s2;
   static unsigned char aux;

	restart_WDT();					//zera WDT

	//reinicia timer1 com 3096, mais contagem que passou
	set_timer1(3096+get_timer1());
	conta++;
	if (conta == 2){
		conta=0;

      aux = valor_disp[0] * 10;    //pega primeiro valor e transforma
      aux = aux + valor_disp[1];

      if (aux>=temp1){             //compara para ver se acende LED1
		   s1=!s1;
		   output_bit(LED1,s1);
      }
      else
         output_low(LED1);

      aux = valor_disp[2] * 10;    //pega segundo valor e transforma
      aux = aux + valor_disp[3];

      if (aux>=temp2){             //compara para ver se acende LED1
		   s2=!s2;
		   output_bit(LED2,s2);
      }
      else
         output_low(LED2);

	}

	restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo para escrever nos displays
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void mostra_disp(){

   unsigned char i;

   for (i=0;i<4;i++){

      output_low(DISP1);         //desliga todos os catodos
      output_low(DISP2);
      output_low(DISP3);
      output_low(DISP4);

      DISPLAY=0;                  //zera segmentos (desliga todos)

      if ((valor_disp[i] > 9) || (valor_disp[i] < 0))
         valor_disp[i]=0x00;

      DISPLAY = segmentos[valor_disp[i]];  //mostra nmero solicitado

      switch (i){
         case 0:
            output_high(DISP1);
            break;
         case 1:
            output_high(DISP2);
            break;
         case 2:
            output_high(DISP3);
            break;
         case 3:
            output_high(DISP4);
            break;
      }

      restart_WDT();              //zera WDT
      delay_ms(5);                //aguarda
   }

   restart_WDT();                //zera WDT

}

//******************************************************************************
// Funo principal do mdulo
//******************************************************************************
void main(void) {

  	OSCCON=0x60;			//configura clock interno para 4MHz

 	set_tris_A(0x21);   //configura pinos do port A como sada, exceto RA0 e RA5
   set_tris_B(0x00);   //configura pinos do port B como sada

	setup_wdt(WDT_2304MS);			//habilita watch dog timer a cada 2 segundos
	restart_WDT();						//zera WDT

   //desliga comparadores
   setup_ccp1(CCP_OFF);
   setup_ccp2(CCP_OFF);

   //timer 0 habilitado
   //[(4MHz/4)/64]/125 = 125 Hz -> valor da frequencia escolhida
   //para obter o valor de inicializao do timer deve-se fazer (256-125)=131
   setup_timer_0(RTCC_INTERNAL | RTCC_DIV_64);
   set_timer0(131);

   //timer 1 habilitado
	// [(4MHz/4)/8]/65636 = 1.90Hz -> menor frequencia possvel
	//valor inicial do timer1 (65536-62500)=3036
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(3036);
	restart_WDT();								//zera WDT

	//timer2 desabilitado
	setup_timer_2(T2_DISABLED,0,1);
	restart_WDT();									//zera WDT

	//habilita interrupes
   enable_interrupts(INT_TIMER0);   //habilita interrupo do timer0
   enable_interrupts(INT_TIMER1);   //habilita interrupo do timer1
  	enable_interrupts(GLOBAL);			//habilita interrupo global
   restart_WDT();							//zera WDT

	mostra_DISP();							//apaga todos os displays
	restart_WDT();							//zera WDT

   setup_ADC_ports(RA0_analog);		//configura porta analgica
	setup_adc(ADC_CLOCK_INTERNAL);	//configura clock para conversor AD
	set_adc_channel(0);					//seta porta
	restart_WDT();							//zera WDT
	delay_ms(1000);						//espera 1 segundo para inicializar
	restart_WDT();							//zera WDT

   output_low(LED1);                //desliga LEDs
   output_low(LED2);

   LM35_0_1=1;

   while(TRUE)
      mostra_disp();               //escreve valores no display
}
