//******************************************************************************
// Arquivo termometro.h
// Desenvolvido por Mrcio Jos Soares
// diretrizes de pr-compilao para mdulo termometro.c
//******************************************************************************

//******************************************************************************
// Arquivos includos
//******************************************************************************
#include <16F819.h>

//******************************************************************************
// Definio para o microcontrolador
//******************************************************************************
#device adc=10									//nmero de bits na converso
#use delay(clock=4000000)					//clock

//******************************************************************************
// Seta perifricos do microcontrolador
//******************************************************************************
// INTRC_IO		   - oscilador interno 8MHz
// WDT 				- habilita WatchDog Timer
// NOLVP 			- desabilita gravao com baixa tenso
// NOPUT 			- desabilita Power Up Timer
// NOBROWNOUT	   - desabilita Brown Out Detect
// NOMCLR 		   - desabilita reset externo. Pino de reset passa a ser I/O
// NOWRT 			- desabilita proteo contra escrita na rea de programa
// NOCPD 			- desabilita proteo contra escrita na rea de dados
// CCPB2 			- canal CCP setado no pino RB2
#fuses INTRC_IO, WDT, NOLVP, NOPUT, NOBROWNOUT, NOMCLR, NOWRT, NOCPD

#byte OSCCON	= 0x8F					//endereo do registrador que configura
												//o clock interno

#byte PORTB    = 0x06               //endereo do registrador do port B

// Pinos utilizados:
// RA0 - AD 1 (entrada para LM35 aps amplificao)
// RA1 - AD 2 (entrada para LM35 aps amplificao)
// RA2 - controle catodo DISP4
// RA3 - controle catodo DISP3
// RA4 - controle catodo DISP2
// RA5 - jumper JP1
// RA6 - controle LED1 (indica que preset para temp1 alcanada)
// RA7 - controle LED2 (indica que preset para temp2 alcanada)
// RB0 - controle catodo DISP1
// RB1 - segmento A para displays
// RB2 - segmento B para displays
// RB3 - segmento C para displays
// RB4 - segmento D para displays
// RB5 - segmento E para displays
// RB6 - segmento F para displays
// RB7 - segmento G para displays

//******************************************************************************
// Definies para o mdulo
//******************************************************************************
#define DISPLAY  	PORTB             //porta para displays de sete segmentos

#define DISP1     PIN_B0            //controle do catodo para DISP1
#define DISP2     PIN_A4            //controle do catodo para DISP2
#define DISP3     PIN_A3            //controle do catodo para DISP3
#define DISP4     PIN_A2            //controle do catodo para DISP4
#define LED1      PIN_A6            //LED1 (indica que preset para temp1 alcanada)
#define LED2      PIN_A7            //LED2 (indica que preset para temp2 alcanada)
#define LM35	   PIN_A0            //entrada analgica para primeiro LM35
#define L4066     PIN_A1            //controle do 4066

#define JP1			PIN_A5				//jumper 1 -

//******************************************************************************
// Constantes do mdulo (tabela para converso dos segmentos)
// valores a serem inseridos nos displays sete segmentos
//******************************************************************************
const unsigned char segmentos[]={0x7E, /* 0 */
                                 0x0C, /* 1 */
                                 0xB6, /* 2 */
                                 0x9E, /* 3 */
                                 0xCC, /* 4 */
                                 0xDA, /* 5 */
                                 0xFA, /* 6 */
                                 0x0E, /* 7 */
                                 0xFE, /* 8 */
                                 0xCE  /* 9 */
                                 };
//******************************************************************************
// Variveis globais do mdulo
//******************************************************************************
int16 valorADC;                              //guarda valor lido pelo AD
unsigned char LM35_0_1;                      //determina qual LM35 deve ler
unsigned char valor_disp[]={0x00, 0x00, 0x00, 0x00};   //valores para displays
unsigned char temp1 = 31;                    //temperatura limite p/ sensor 1
unsigned char temp2 = 31;                    //temperatura limite p/ sensor 2
